@@ -11,21 +11,29 @@ use Cwd qw(abs_path);
 use English qw( $EVAL_ERROR $OSNAME $PERL_VERSION -no_match_vars );
 use File::Basename qw(dirname);
 use File::Spec::Functions qw( abs2rel catfile );
+use Test::Builder;
 use Test::Net::SSLeay::Socket;
 
-our $VERSION = '1.90';
+our $VERSION = '1.94';
 
 our @EXPORT_OK = qw(
     can_fork can_really_fork can_thread
     data_file_path
+    dies_like
+    dies_ok
+    doesnt_warn
     initialise_libssl
     is_libressl is_openssl
     is_protocol_usable
+    lives_ok
     new_ctx
     protocols
     tcp_socket
+    warns_like
 );
 
+my $tester = Test::Builder->new();
+
 my $data_path = catfile( dirname(__FILE__), '..', '..', '..', 't', 'data' );
 
 my $initialised = 0;
@@ -63,6 +71,29 @@ my %protos = (
     },
 );
 
+my ( $test_no_warnings, $test_no_warnings_name, @warnings );
+
+END {
+    _test_no_warnings() if $test_no_warnings;
+}
+
+sub _all {
+    my ( $sub, @list ) = @_;
+
+    for (@list) {
+        $sub->() or return 0;
+    }
+
+    return 1;
+}
+
+sub _diag {
+    my (%args) = @_;
+
+    $tester->diag( ' ' x 9, 'got: ', $args{got} );
+    $tester->diag( ' ' x 4, 'expected: ', $args{expected} );
+}
+
 sub _libssl_fatal {
     my ($context) = @_;
 
@@ -76,6 +107,17 @@ sub _load_net_ssleay {
     return 1;
 }
 
+sub _test_no_warnings {
+    my $got_str = join q{, }, map { qq{'$_'} } @warnings;
+    my $got_type = @warnings == 1 ? 'warning' : 'warnings';
+
+    $tester->ok( @warnings == 0, $test_no_warnings_name )
+        or _diag(
+            got      => "$got_type $got_str",
+            expected => 'no warnings',
+        );
+}
+
 sub import {
     my ( $class, @imports ) = @_;
 
@@ -126,9 +168,13 @@ sub can_thread {
     # (see GH #175)
     if (    $PERL_VERSION == 5.010000
         and $Config{ccname} eq 'gcc'
-        and $Config{gccversion} )
+        and defined $Config{gccversion}
+        # gccversion is sometimes defined for non-GCC compilers (see GH-350);
+        # compilers that are truly GCC are identified with a version number in
+        # gccversion
+        and $Config{gccversion} =~ /^\d+\.\d+/ )
     {
-        my ( $gcc_major, $gcc_minor ) = split /\./, $Config{gccversion};
+        my ( $gcc_major, $gcc_minor ) = split /[.\s]+/, $Config{gccversion};
 
         return 0
             if ( $gcc_major > 4 or ( $gcc_major == 4 and $gcc_minor >= 8 ) );
@@ -152,6 +198,69 @@ sub data_file_path {
     return $rel_path;
 }
 
+sub dies_like {
+    my ( $sub, $expected, $name ) = @_;
+
+    my ( $got, $ok );
+
+    if ( eval { $sub->(); 1 } ) {
+        $ok = $tester->ok ( 0, $name );
+
+        _diag(
+            got      => 'subroutine lived',
+            expected => "subroutine died with exception matching $expected",
+        );
+    }
+    else {
+        $got = $EVAL_ERROR;
+
+        my $test = $got =~ $expected;
+
+        $ok = $tester->ok( $test, $name )
+            or _diag(
+                got      => qq{subroutine died with exception '$got'},
+                expected => "subroutine died with exception matching $expected",
+            );
+    }
+
+    $EVAL_ERROR = $got;
+
+    return $ok;
+}
+
+sub dies_ok {
+    my ( $sub, $name ) = @_;
+
+    my ( $got, $ok );
+
+    if ( eval { $sub->(); 1 } ) {
+        $got = $EVAL_ERROR;
+
+        $ok = $tester->ok ( 0, $name );
+
+        _diag(
+            got      => 'subroutine lived',
+            expected => 'subroutine died',
+        );
+    }
+    else {
+        $got = $EVAL_ERROR;
+
+        $ok = $tester->ok( 1, $name );
+    }
+
+    $EVAL_ERROR = $got;
+
+    return $ok;
+}
+
+sub doesnt_warn {
+    $test_no_warnings      = 1;
+    $test_no_warnings_name = shift;
+
+    $SIG{__WARN__} = sub { push @warnings, shift };
+}
+
 sub initialise_libssl {
     return 1 if $initialised;
 
@@ -304,6 +413,32 @@ sub is_protocol_usable {
     croak 'Unexpected TLS state machine sequence: ' . join( ', ', @states );
 }
 
+sub lives_ok {
+    my ( $sub, $name ) = @_;
+
+    my ( $got, $ok );
+
+    if ( !eval { $sub->(); 1 } ) {
+        $got = $EVAL_ERROR;
+
+        $ok = $tester->ok ( 0, $name );
+
+        _diag(
+            got      => qq{subroutine died with exception '$got'},
+            expected => 'subroutine lived',
+        );
+    }
+    else {
+        $got = $EVAL_ERROR;
+
+        $ok = $tester->ok( 1, $name );
+    }
+
+    $EVAL_ERROR = $got;
+
+    return $ok;
+}
+
 sub new_ctx {
     my ( $min_proto, $max_proto ) = @_;
 
@@ -362,6 +497,41 @@ sub tcp_socket {
     return Test::Net::SSLeay::Socket->new( proto => 'tcp' );
 }
 
+sub warns_like {
+    my ( $sub, $expected, $name ) = @_;
+
+    my @expected =   ref $expected eq 'ARRAY'
+                   ? @$expected
+                   : ($expected);
+
+    my @got;
+
+    local $SIG{__WARN__} = sub { push @got, shift };
+
+    $sub->();
+
+    $SIG{__WARN__} = 'DEFAULT';
+
+    my $test =    scalar @got == scalar @expected
+               && _all( sub { $got[$_] =~ $expected[$_] }, 0 .. $#got );
+
+    my $ok = $tester->ok( $test, $name )
+        or do {
+            my $got_str      = join q{, }, map { qq{'$_'} } @got;
+            my $expected_str = join q{, }, map { qq{'$_'} } @expected;
+
+            my $got_plural      = @got == 1 ? '' : 's';
+            my $expected_plural = @expected == 1 ? '' : 's';
+
+            _diag(
+                got      => "warning$got_plural $got_str",
+                expected => "warning$expected_plural matching $expected_str",
+            );
+        };
+
+    return $ok;
+}
+
 1;
 
 __END__
@@ -372,7 +542,7 @@ Test::Net::SSLeay - Helper module for the Net-SSLeay test suite
 
 =head1 VERSION
 
-This document describes version 1.90 of Test::Net::SSLeay.
+This document describes version 1.94 of Test::Net::SSLeay.
 
 =head1 SYNOPSIS
 
@@ -466,6 +636,49 @@ this Perl, or false if not.
 Returns the relative path to a given file in the test suite data directory
 (C<t/local/>). Dies if the file does not exist.
 
+=head2 dies_like
+
+    dies_like(
+        sub { die 'This subroutine always dies' },
+        qr/always/,
+        'A test that always passes'
+    );
+
+Similar to L<C<throws_ok> in Test::Exception|Test::Exception/throws_ok>:
+performs a L<Test::Builder> test that passes if a given subroutine dies with an
+exception string that matches a given pattern, or fails if the subroutine does
+not die or dies with an exception string that does not match the given pattern.
+
+This function preserves the value of C<$@> set by the given subroutine, so (for
+example) other tests can be performed on the value of C<$@> afterwards.
+
+=head2 dies_ok
+
+    dies_ok(
+        sub { my $x = 1 },
+        'A test that always fails'
+    );
+
+Similar to L<C<dies_ok> in Test::Exception|Test::Exception/dies_ok>: performs a
+L<Test::Builder> test that passes if a given subroutine dies, or fails if it
+does not.
+
+This function preserves the value of C<$@> set by the given subroutine, so (for
+example) other tests can be performed on the value of C<$@> afterwards.
+
+=head2 doesnt_warn
+
+    doesnt_warn('Test script outputs no unexpected warnings');
+
+Offers similar functionality to L<Test::NoWarnings>: performs a L<Test::Builder>
+test at the end of the test script that passes if the test script executes from
+this point onwards without emitting any unexpected warnings, or fails if
+warnings are emitted before the test script ends.
+
+Warnings omitted by subroutines that are executed as part of a L</warns_like>
+test are not considered to be unexpected (even if the L</warns_like> test
+fails), and will therefore not cause this test to fail.
+
 =head2 initialise_libssl
 
     initialise_libssl();
@@ -514,6 +727,20 @@ C<no-ssl3>), or run-time configuration (e.g., the use of TLSv1.0 will be
 forbidden if the OpenSSL configuration sets the default security level to 3 or
 higher; see L<SSL_CTX_set_security_level(3)>).
 
+=head2 lives_ok
+
+    lives_ok(
+        sub { die 'Whoops' },
+        'A test that always fails'
+    );
+
+Similar to L<C<lives_ok> in Test::Exception|Test::Exception/lives_ok>: performs
+a L<Test::Builder> test that passes if a given subroutine executes without
+dying, or fails if it dies during execution.
+
+This function preserves the value of C<$@> set by the given subroutine, so (for
+example) other tests can be performed on the value of C<$@> afterwards.
+
 =head2 new_ctx
 
     my $ctx = new_ctx();
@@ -566,6 +793,27 @@ closing connections.
 Returns a L<Test::Net::SSLeay::Socket|Test::Net::SSLeay::Socket> object. Dies
 on failure.
 
+=head2 warns_like
+
+    warns_like(
+        sub {
+            warn 'First warning';
+            warn 'Second warning';
+        },
+        [
+            qr/First/,
+            qr/Second/,
+        ],
+        'A test that always passes'
+    );
+
+Similar to L<C<warnings_like> in Test::Warn|Test::Warn/warnings_like>: performs
+a L<Test::Builder> test that passes if a given subroutine emits a series of
+warnings that match the given sequence of patterns, or fails if the subroutine
+emits any other sequence of warnings (or no warnings at all). If a pattern is
+given instead of an array reference, the subroutine will be expected to emit a
+single warning matching the pattern.
+
 =head1 BUGS
 
 If you encounter a problem with this module that you believe is a bug, please
@@ -603,14 +851,12 @@ the version of OpenSSL or LibreSSL you are using.
 
 Originally written by Chris Novakovic.
 
-Maintained by Chris Novakovic, Tuure Vartiainen and Heikki Vatiainen.
+Maintained by Chris Novakovic and Heikki Vatiainen.
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2020- Chris Novakovic <chris@chrisn.me.uk>.
 
-Copyright 2020- Tuure Vartiainen <vartiait@radiatorsoftware.com>.
-
 Copyright 2020- Heikki Vatiainen <hvn@radiatorsoftware.com>.
 
 This module is released under the terms of the Artistic License 2.0. For
